/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.dynamicprocesses;


import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 * An example on creating dynamic processes. During its life, every process creates four sons and activates them. The sons have the same
 * name as their father but with an extra letter (A, B, C or D) at the end. The level of recursion is limited by 4.
 * 
 */
public class DynamicProcessesExample
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		DynamicProcess dp = null;

		try
		{
			// First, we create a simulation and one process.
			simulation = new JSimSimulation("Dynamic Processes Example Simulation");
			// This is the father of all other processes
			dp = new DynamicProcess("X", simulation, 0);

			// We must not forget to activate the process. It will create and activate the others.
			dp.activate(0.0);

			// Now we run the simulation until there are no processes scheduled.
			while (simulation.step() == true)
				;

			simulation.message("It's over.");
		} // try

		// All three possible exceptions are caught here in one handler.
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// It is a good practice to put the shutdown in the finally block. Then you can be sure that that the JVM will not "hang".
		finally
		{
			simulation.shutdown();
		}
	} // main

} // DynamicProcessesExample
