/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.systemutils.randomuserobjects;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.random.JSimUserObjectStream;

/**
 * This example shows how to use the random generator of user-specified objects.
 * 
 */
public class FruitOfTheLoom
{
	public static final int NO_OF_FRUIT_SPECIES = 5;

	public static void main(String[] args)
	{
		int i;
		JSimUserObjectStream s;
		Object[] fruit;
		double[] fruitRates;
		Object randomObject;

		int noOfApples = 0;
		int noOfPears = 0;
		int noOfOranges = 0;
		int noOfBananas = 0;
		int noOfTomatoes = 0;

		try
		{
			fruit = new Object[NO_OF_FRUIT_SPECIES];
			fruit[0] = new Apple();
			fruit[1] = new Pear();
			fruit[2] = new Orange();
			fruit[3] = new Banana();
			fruit[4] = new Tomato();

			fruitRates = new double[NO_OF_FRUIT_SPECIES];
			fruitRates[0] = 20.0;
			fruitRates[1] = 15.0;
			fruitRates[2] = 35.0;
			fruitRates[3] = 20.0;
			fruitRates[4] = 10.0;

			s = new JSimUserObjectStream(fruit, fruitRates);
			for (i = 0; i < 1000; i++)
			{
				randomObject = s.getNext();
				System.out.print(randomObject + " ");

				if (randomObject == fruit[0])
					noOfApples++;
				if (randomObject == fruit[1])
					noOfPears++;
				if (randomObject == fruit[2])
					noOfOranges++;
				if (randomObject == fruit[3])
					noOfBananas++;
				if (randomObject == fruit[4])
					noOfTomatoes++;
			} // for i

			System.out.println();
			System.out.println();
			System.out.println("------------");
			System.out.println();
			System.out.println("Number of apples:   " + noOfApples + ". Should be around 200 of 1000.");
			System.out.println("Number of pears:    " + noOfPears + ". Should be around 150 of 1000.");
			System.out.println("Number of oranges:  " + noOfOranges + ". Should be around 350 of 1000.");
			System.out.println("Number of bananas:  " + noOfBananas + ". Should be around 200 of 1000.");
			System.out.println("Number of tomatoes: " + noOfTomatoes + ". Should be around 100 of 1000.");
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // main

} // class FruitOfTheLoom

class Apple
{
	public String toString()
	{
		return "Apple";
	} // toString
} // class Apple

class Pear
{
	public String toString()
	{
		return "Pear";
	} // toString
} // class Pear

class Orange
{
	public String toString()
	{
		return "Orange";
	} // toString
} // class Orange

class Banana
{
	public String toString()
	{
		return "Banana";
	} // toString
} // class Banana

class Tomato
{
	public String toString()
	{
		return "Tomato";
	} // toString
} // class Tomato
