/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.queues.queueexample1;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 *  A simple example on using queues. We insert seven items in the queue and then print them out.
 *
 */
public class QueueExample1
{
	public static void main(String[] args)
	{
		JSimSimulation sim = null;
		JSimHead queue;
		JSimLink qi, after, before;
		int i;

		try
		{
			// It is necessary to create a simulation before creating a queue -- the queue needs simulation's time functions.
			sim = new JSimSimulation("Simulation with queues");
			queue = new JSimHead("Little queue", sim);

			// Let's insert some data into the queue...
			for (i = 0; i < 5; i++)
			{
				qi = new JSimLink(new Integer(i));
				qi.into(queue);
			} // for

			sim.message("Number of items in the queue: " + queue.cardinal());
			sim.message("Is the queue empty? " + queue.empty());

			// Now we insert two other items into the queue -- one after the first item and one before the last one.
			after = new JSimLink(new Float(0.5));
			before = new JSimLink(new Float(3.5));

			after.follow(queue.first());
			before.precede(queue.last());

			sim.message("Number of items in the queue: " + queue.cardinal());
			sim.message("Is the queue empty? " + queue.empty());

			// 0 0.5 1 2 3 3.5 4

			// Now, we will print out all the data stored in the queue, taking them out of the queue.
			while (!queue.empty())
			{
				qi = queue.first();
				sim.message("The first item is of type: " + qi.getDataType());
				sim.message("Data: " + qi.getData());
				
				// Take it out!
				qi.out();
				sim.message("Number of items in the queue: " + queue.cardinal());
			} // while

			sim.message("Clearing the queue...");
			queue.clear();
		} // try

		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch

		// It is not necessary to shutdown the simulation when there are no processes, but maybe a process will be added later and you will forget to do it.
		finally
		{
			sim.shutdown();
		} // finally
	} // main

} // class QueueExample1
