/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.queues.queueexample2;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 * A simple example on using queues. We will put 100.000 numbers into the queue, print them out and then release them from the queue.
 * 
 */
public class QueueExample2
{
	public static void main(String[] args)
	{
		final int noOfElements = 100000;
		
		JSimSimulation sim = null;
		JSimHead queue;
		JSimLink qi;
		int i, j;
		long time1, time2;
		long t1, t2, t3, t4;
		
		try
		{
			// It is necessary to create a simulation before creating a queue -- the queue needs simulation's time functions.
			sim = new JSimSimulation("Simulation with queues");
			queue = new JSimHead("Big queue", sim);

			// Let's insert some data into the queue...
			time1 = System.currentTimeMillis();

			for (i = 0; i < noOfElements; i++)
			{
				qi = new JSimLink(new Integer(i));
				qi.into(queue);
			} // for
			time2 = System.currentTimeMillis();
			t1 = time2 - time1;

			sim.message("Number of items in the queue: " + queue.cardinal());
			sim.message("Is the queue empty? " + queue.empty());

			// Now, we will print out all the data stored in the queue, without taking them out of the queue.
			sim.message("The queue contains these data:");
			time1 = System.currentTimeMillis();
			qi = queue.first();
			j = 0;
			while (qi != null)
			{
				sim.messageNoNL(" " + qi.getData());
				qi = qi.getNext();
				if ((++j % 10) == 0)
					sim.message("");
			} // while
			time2 = System.currentTimeMillis();
			t2 = time2 - time1;

			// The same work again without printing the numbers
			time1 = System.currentTimeMillis();
			qi = queue.first();
			while (qi != null)
			{
				qi = qi.getNext();
			} // while
			time2 = System.currentTimeMillis();
			t3 = time2 - time1;

			// Now we release all the item from the queue.
			sim.message("");
			sim.message("Clearing the queue...");
			time1 = System.currentTimeMillis();
			queue.clear();
			time2 = System.currentTimeMillis();
			t4 = time2 - time1;

			sim.message("The queue contains these data:");
			qi = queue.first();
			j = 0;
			while (qi != null)
			{
				sim.messageNoNL(" " + qi.getData());
				qi = qi.getNext();
				if ((++j % 10) == 0)
					sim.message("");
			} // while

			sim.message("-------------------------------------------------------------------------");
			sim.message("Inserting " + noOfElements + " links: " + t1 + " ms.");
			sim.message("Getting " + noOfElements + " links with printing: " + t2 + " ms.");
			sim.message("Getting " + noOfElements + " links without printing: " + t3 + " ms.");
			sim.message("Removing " + noOfElements + " links: " + t4 + " ms.");
		} // try

		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		}

		// It is not necessary to shutdown the simulation when there are no processes, but maybe a process will be added later and you will forget to do it.
		finally
		{
			sim.shutdown();
		} // finally
	} // main

} // class QueueExample2
