/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.queues.producerconsumer;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * Periodicly gets data from the queue.
 */ 
public class ConsumerProcess extends JSimProcess
{
	private JSimHead queue;
	private double lambda;

	/**
	 * It is better to throw the exceptions from the constructor than to catch them.
	 */ 
	public ConsumerProcess(String name, JSimSimulation simulation, JSimHead q, double l) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, simulation);
		if (q == null)
			throw new JSimInvalidParametersException("ConsumerProcess.ConsumerProcess(): q");
		queue = q;
		lambda = l;
	} // constructor

	/**
	 * We don't care about stopping &ndash; we run infinitely.
	 */ 
	protected void life()
	{
		try
		{
			JSimLink link;
			double time;

			// Yes, it's really a neverending cycle
			while (true)
			{
				time = myParent.getCurrentTime();
				// If the queue is empty, we don't do anything.
				if (queue.empty())
					message(time + " -- " + getName() + ": queue empty.");
				else
				{
					// We must pick up the first item and take it out from the queue.
					link = queue.first();
					link.out();
					message(time + " -- " + getProcessName() + ": received " + link.getData());
				} // else

				hold(JSimSystem.negExp(lambda));
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
	} // life

} // class ConsumerProcess
