/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.queues.producerconsumer;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * Periodicly puts data into the queue.
 */ 
public class ProducerProcess extends JSimProcess
{
	private JSimHead queue;
	private double lambda;

	/**
	 * It is better to throw the exceptions from the constructor than to catch them.
	 */ 
	public ProducerProcess(String name, JSimSimulation simulation, JSimHead q, double l) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, simulation);
		if (q == null)
			throw new JSimInvalidParametersException("ProducerProcess.ProducerProcess(): q");
		queue = q;
		lambda = l;
	} // constructor

	/**
	 * We don't care about stopping &ndash; we run infinitely.
	 */ 
	protected void life()
	{
		try
		{
			JSimLink link;
			double time;
			int number;

			// Yes, it's really a neverending cycle
			while (true)
			{
				time = myParent.getCurrentTime();
				number = Math.round(100.0F * (float) Math.random());
				message(time + " -- " + getName() + ": producing " + number);

				// We create a new item and insert it into the queue.
				link = new JSimLink(new Integer(number));
				link.into(queue);
				hold(JSimSystem.negExp(lambda));
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
	} // life

} // class ProducerProcess
