/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *  Copyright (c) 2003 Pavel Domecký
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.gui.changeoutputinfo.custominfo;

import java.util.Collection;

import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimTooManyHeadsException;
import cz.zcu.fav.kiv.jsim.gui.JSimPair;

/**
 * A queue having its own server which takes transactions out from the queue.
 * 
 * Methods getObjectListItemDescription() and getDetailedInformationArray() are overridden with our own code which modifies the set of
 * characteristics usually shown.
 */
public class QueueWithServer extends JSimHead
{
	private JSimProcess server;

	public QueueWithServer(String name, JSimSimulation sim, JSimProcess s) throws JSimInvalidParametersException, JSimTooManyHeadsException
	{
		super(name, sim);
		server = s;
	} // constructor

	public JSimProcess getServer()
	{
		return server;
	} // getServer

	public void setServer(JSimProcess s)
	{
		server = s;
	} // setServer

	/**
	 * Overrides the method from class JSimHead. Now the detailed info window displays the average length of the queue in addition to the
	 * standard information.
	 */
	public String getObjectListItemDescription()
	{
		return ("No.: " + getHeadNumber() + "  Name: " + getHeadName() + "  Elements: " + cardinal() + "  Average length: " + getLw());
	}

	/**
	 * Overrides the method from class JSimHead. A new row is added to the table: the average waiting time of all elements in the queue.
	 */
	public Collection<JSimPair> getDetailedInformationArray()
	{
		Collection<JSimPair> c;
		c = super.getDetailedInformationArray();
		c.add(new JSimPair("Average waiting time for all links:", Double.toString(getTwForAllLinks())));
		return (c);
	}

} // class QueueWithServer
