/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *  Copyright (c) 2003 Pavel Domecký
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.gui.changeoutputinfo.inheritedinfowindow;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.util.Observable;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;

import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.gui.JSimDetailedInfoWindow;
import cz.zcu.fav.kiv.jsim.gui.JSimMainWindow;

/**
 * If we dont't want to use the default detailed info window that J-Sim offers, it's possible to create our own window inherited from
 * JSimGUIDetailedInfoWindow and to show whatever we want in this window.
 * 
 * The MyGUIDetailedInfoWindow class provides a dialog showing a panel with a text.
 */
public class MyGUIDetailedInfoWindow extends JSimDetailedInfoWindow
{
	private static final long serialVersionUID = 7771957249368711006L;

	/**
	 * Panel showing our own characteristics.
	 */ 
	class TextPanel extends JPanel
	{
		private static final long serialVersionUID = 6220020904977238140L;

		public TextPanel()
		{
			setPreferredSize(new Dimension(200, 100));
			Border raisedBevel = BorderFactory.createRaisedBevelBorder();
			Border loweredBevel = BorderFactory.createLoweredBevelBorder();
			Border compound = BorderFactory.createCompoundBorder(raisedBevel, loweredBevel);
			setBorder(compound);
			setFont(new Font("SansSerif", Font.BOLD, 30));
		}

		/**
		 * In this panel we show the number of elements in the queue (in graphic mode)
		 */ 
		public void paintComponent(Graphics g)
		{
			super.paintComponent(g);
			g.drawString(Long.toString(((JSimHead) objectInfo).cardinal()), getWidth() / 2 - 10, getHeight() / 2);
		}
	} // inner class TextPanel

	TextPanel panelSpec;

	/**
	 * Just use parent constructor.
	 */ 
	public MyGUIDetailedInfoWindow(JSimMainWindow parent, JSimHead info)
	{
		super(parent, info);
	}

	/**
	 * Overriding the method of its parent. This is the key action you have to do when replacing the content of the info window!
	 */
	protected void constructorSpecificInit()
	{
		panelSpec = new TextPanel();
		getContentPane().add(new JLabel("Number of elements:"), BorderLayout.NORTH);
		getContentPane().add(panelSpec, BorderLayout.CENTER);
	}

	/**
	 *  Overriding the method of its parent.
	 */
	public void update(Observable o, Object arg)
	{
		panelSpec.repaint();
	} // update

} // MyGUIDetailedInfoWindow
