/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *  Copyright (c) 2003 Pavel Domecký
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.gui.changeoutputinfo.newinfowindow;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.Border;

import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimProcessState;
import cz.zcu.fav.kiv.jsim.gui.JSimMainWindow;

/**
 * This class is a replacement for the default J-Sim info window. It does not only replaces the information content of the window but the
 * whole window.
 * 
 * MyGUIDetailedInfoWindow class provides a dialog which shows a panel with text.
 */
public class MyGUIDetailedInfoWindow extends JDialog implements Observer
{
	private static final long serialVersionUID = 4149979702710587249L;

	private JSimMainWindow myParent;
	private JPanel panel;
	private TextPanel panelSpec;
	private JButton button;
	private JSimProcess objectInfo;
	private JScrollPane panelSpecScrollPane;

	/**
	 * Dialog's window adapter.
	 * 
	 * Although we could implement it directly in the main window we rather create an extra class. We must implement ActionListener since
	 * there is no ActionAdapter to be possibly extended. We must not forget to remove object (whose information we display here) from
	 * hashtable in JSimGUIMainWindowSwing.
	 */
	class JSimGUIDetailedInfoWindowWindowAdapter extends WindowAdapter
	{
		public void windowClosing(WindowEvent e)
		{
			myParent.removeDisplayableFromOpenInfoWindows(objectInfo);
			setVisible(false);
			dispose();
		} // windowClosing
	} // class JSimGUIDetailedInfoWindowWindowAdapter

	/**
	 * Dialog's action adapter.
	 * 
	 * Although we could implement it directly in the main window we rather create an extra class. We must implement ActionListener since
	 * there is no ActionAdapter to be possibly extended. We must not to forget to remove object (whose information we display here) from
	 * hashtable in JSimGUIMainWindowSwing.
	 */
	class JSimGUIDetailedInfoWindowActionAdapter implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			myParent.removeDisplayableFromOpenInfoWindows(objectInfo);
			setVisible(false);
			dispose();
		} // actionPerformed
	} // class JSimGUIDetailedInfoWindowActionAdapter

	/**
	 * Panel showing our own characteristics.
	 */
	class TextPanel extends JPanel
	{
		private static final long serialVersionUID = 1025000218768492908L;

		public TextPanel()
		{
			setPreferredSize(new Dimension(350, 100));
			Border raisedBevel = BorderFactory.createRaisedBevelBorder();
			Border loweredBevel = BorderFactory.createLoweredBevelBorder();
			Border compound = BorderFactory.createCompoundBorder(raisedBevel, loweredBevel);
			setBorder(compound);
			setFont(new Font("SansSerif", Font.BOLD, 20));
		}

		/**
		 * In this panel we show process state (in graphics).
		 */
		public void paintComponent(Graphics g)
		{
			super.paintComponent(g);
			if (objectInfo.getProcessState() == JSimProcessState.SCHEDULED)
				g.drawString(objectInfo.getProcessState().toString() + " for " + objectInfo.getScheduleTime(), 10, getHeight() / 2);
			else
				g.drawString(objectInfo.getProcessState().toString(), 10, getHeight() / 2);
		}
	} // class TextPanel

	/**
	 * Creates a new dialog containing a panel (showing detailed information of object) and an OK button.
	 * 
	 * @param parent
	 *            Parent window of the dialog.
	 * @param info
	 *            Process whose informations are about to be displayed in the panel.
	 */
	public MyGUIDetailedInfoWindow(JSimMainWindow parent, JSimProcess info)
	{
		super(parent, "My own window", false);
		myParent = parent;
		objectInfo = info;
		panel = new JPanel();
		button = new JButton("OK");
		panel.add(button);
		panelSpec = new TextPanel();
		panelSpecScrollPane = new JScrollPane(panelSpec);

		getContentPane().add(new JLabel("Process state:"), "North");
		getContentPane().add(panelSpecScrollPane, "Center");
		getContentPane().add(panel, "South");

		button.addActionListener(new JSimGUIDetailedInfoWindowActionAdapter());
		addWindowListener(new JSimGUIDetailedInfoWindowWindowAdapter());
		
		// We must call update to fill the table model.
		Observable a = null;
		Object b = null;
		update(a, b);

		pack();
		setLocationRelativeTo(null);
	} // constructor

	/**
	 * Updates the table model.
	 */
	public void update(Observable o, Object arg)
	{
		panelSpec.repaint();
	} // update

} // class MyGUIDetailedInfoWindow
