/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.performancetesting.maxnumberofprocesses;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * The aim of this program is to find the maximum number of J-Sim processes that can be created on your system. This number depends on your
 * hardware, OS and Java runtime environment.
 */
public class MaxNumberTesting
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		TestingProcess process = null;
		int i = 0;
		int max = 0;

		try
		{
			System.out.println("---------------------------------");
			System.out.println("Testing " + JSimSystem.getVersion());
			System.out.println("---------------------------------");
			System.getProperties().list(System.out);
			System.out.println("---------------------------------");

			// First, we create a simulation and processes.
			simulation = new JSimSimulation("Max Number of Processes Testing Simulation");

			try
			{
				process = new TestingProcess("Process #" + Integer.toString(i), simulation);
				process.activate(0.0);
				i = 1;
				while (simulation.step() == true)
				{
					process = new TestingProcess("Process #" + Integer.toString(i), simulation);
					process.activate(simulation.getCurrentTime() + 0.01);
					i++;
				} // while
			} // try
			catch (OutOfMemoryError err)
			{
				max = i - 1;
				// err.printStackTrace();
			} // catch

			System.out.println("Your system is able to create " + Integer.toString(max) + " J-Sim processes at this moment.");
		} // try
	
		// All possible exceptions are caught here in one handler.
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch

		// It is a good practice to put the shutdown in the finally block. Then you can be sure that that the JVM will not hang.
		// Every simulation must be shut down at the end of the program, even if an error occurs.
		finally
		{
			simulation.shutdown();
		} // finally
	} // main
	
} // class PerformanceTesting

/**
 * A process does some "useful" work and then suspends itself for a given fixed time.
 */
class TestingProcess extends JSimProcess
{
	/**
	 * Note that you must ALWAYS write your own constructor &ndash; JSimProcess' one has two parameters.
	 */ 
	public TestingProcess(String name, JSimSimulation sim) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, sim);
	} // constructor

	/**
	 * This is the life of the process. We will cycle here infinite number of times, doing some simple calculations.
	 */ 
	protected void life()
	{
		@SuppressWarnings("unused") // Not really used, serves only to have some code (which is certainly eliminated by the compiler).
		int i;

		try
		{
			while (true)
			{
				i = 1 + 1;
				hold(2.0);
			} // for i
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment();
		} // catch
	} // life

} // class MaxTesting
