/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.asynchronousindirect1;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;
import cz.zcu.fav.kiv.jsim.ipc.JSimIndirectMessage;
import cz.zcu.fav.kiv.jsim.ipc.JSimMessageBox;

/**
 * A simple process periodically inserting messages to a shared message box. After every insertion, the process sleeps for a while.
 */
public class SendingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimMessageBox box;

	// ------------------------------------------------------------------------------------------------------------------------------------

	public SendingProcess(String name, JSimSimulation parent, JSimMessageBox messageBox) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		box = messageBox;
	} // constructor

	protected void life()
	{
		JSimIndirectMessage message;
		int counter = 0;
		double sleepTime;

		try
		{
			while (true)
			{
				message = new JSimIndirectMessage(new Integer(counter));
				message(myParent.getCurrentTime() + " - " + getName() + ": Sending a message with data `" + counter + "' to the shared message box.");
				sendMessageWithoutBlocking(message, box);
				message(myParent.getCurrentTime() + " - " + getName() + ": Mesage sent.");
				counter++;
				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class SendingProcess
