/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.asynchronousindirect2;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.ipc.JSimMessageBox;

/**
 * A simple simulation of six processes exchanging messages asynchronously and indirectly via a message box. There are three senders and
 * three receivers. A message sent by S<sub>i</sub> can be read by any R<sub>i</sub>. R<sub>i</sub> can read a message sent by any S<sub>i</sub>.
 * 
 * The messages know neither the sender nor the receiver. Neither the sending nor the receiving operation blocks. If no message is found in
 * the message box by the receiving process, the process just goes on.
 */
public class AsynchronousIndirectCommunication
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		JSimMessageBox box = null;
		SendingProcess s1, s2, s3;
		ReceivingProcess r1, r2, r3;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Asynchronous Indirect Communication Simulation");
			box = new JSimMessageBox("Shared Message Box");
			s1 = new SendingProcess("Sender 1", simulation, box, 'A');
			s2 = new SendingProcess("Sender 2", simulation, box, 'B');
			s3 = new SendingProcess("Sender 3", simulation, box, 'C');
			r1 = new ReceivingProcess("Receiver 1", simulation, box);
			r2 = new ReceivingProcess("Receiver 2", simulation, box);
			r3 = new ReceivingProcess("Receiver 3", simulation, box);

			simulation.message("Activating the processes...");
			s1.activate(0.0);
			s2.activate(0.0);
			s3.activate(0.0);
			r1.activate(0.0);
			r2.activate(0.0);
			r3.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
		finally
		{
			simulation.shutdown();
		} // finally
	} // main

} // class AsynchronousIndirectCommunication
