/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.asynchronoussymmetric;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 * A simple simulation of tree processes exchanging messages asynchronously and directly. The messages know both the sender and the
 * receiver. Neither the sending nor the receiving operation blocks. If no message is found in the message box by the receiving process, the
 * process just goes on. The receiving process tries to read messages from both senders but one of them is never activated so the receiver
 * will be successful in half cases only.
 * 
 * The receiver decides from which process the new message will be received.
 */
public class AsynchronousSymmetricCommunication
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		SendingProcess sender;
		SendingProcess passiveSender;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Asynchronous Symmetric Communication Simulation");
			sender = new SendingProcess("Sender", simulation);
			passiveSender = new SendingProcess("Passive Sender", simulation);
			receiver = new ReceivingProcess("Receiver", simulation, sender, passiveSender);
			sender.setReceiver(receiver);

			simulation.message("Activating the processes...");
			sender.activate(0.0);
			receiver.activate(0.0);
			// Passive receiver is not activated, it will never send a message.

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
		finally
		{
			simulation.shutdown();
		} // finally
	} // main

} // class AsynchronousSymmetricCommunication
