/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.asynchronouswithknownreceiver;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;
import cz.zcu.fav.kiv.jsim.ipc.JSimMessageForReceiver;

/**
 * A simple process periodically sending symmetric messages to a receiving process. After every send, the process sleeps for a while. The
 * receiver must be set before the process is run.
 */
public class SendingProcess extends JSimProcess
{
	public static final double LAMBDA = 0.5;

	private JSimProcess receiver;
	private char prefix;

	// ------------------------------------------------------------------------------------------------------------------------------------

	public SendingProcess(String name, JSimSimulation parent, JSimProcess receivingProcess, char messagePrefix) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		receiver = receivingProcess;
		prefix = messagePrefix;
	} // constructor

	protected void life()
	{
		JSimMessageForReceiver message;
		int counter = 0;
		double sleepTime;
		String messageData;

		try
		{
			while (true)
			{
				messageData = Character.toString(prefix) + Integer.toString(counter);
				message = new JSimMessageForReceiver(receiver, messageData);
				message(myParent.getCurrentTime() + " - " + getName() + ": Sending a message with data `" + messageData + "' to process `" + receiver.getName() + "'.");
				sendMessageWithoutBlocking(message);
				message(myParent.getCurrentTime() + " - " + getName() + ": Mesage sent.");
				counter++;
				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class SendingProcess
