/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.synchronousindirect1;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.ipc.JSimMessageBox;

/**
 * A simple simulation of two processes exchanging messages synchronously and indirectly via a message box. The messages know neither the
 * sender nor the receiver. Both the sending and the receiving operations block. If no message is found in the message box by the receiving
 * process, the process blocks until the sender sends it a message. The sender blocks during send if the receiver is not ready to receive a
 * message. The sender remains blocked until the receiver performs a receive operation.
 */
public class SynchronousIndirectCommunication
{
	public static void main(String[] args)
	{
		JSimSimulation simulation = null;
		JSimMessageBox box;
		SendingProcess sender;
		ReceivingProcess receiver;

		try
		{
			System.out.println("Initializing the simulation...");
			simulation = new JSimSimulation("Synchronous Indirect Communication Simulation");
			box = new JSimMessageBox("Shared Message Box");
			receiver = new ReceivingProcess("Receiver", simulation, box);
			sender = new SendingProcess("Sender", simulation, box);

			simulation.message("Activating the processes...");
			sender.activate(0.0);
			receiver.activate(0.0);

			simulation.message("Running the simulation, please wait.");

			while ((simulation.getCurrentTime() < 100.0) && (simulation.step() == true))
				;
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
		finally
		{
			simulation.shutdown();
		} // finally
	} // main

} // class SynchronousIndirectCommunication
