/*
 *	Copyright (c) 2000-2006 Jaroslav Kačer <jaroslav@kacer.biz>
 *	Licensed under the Academic Free License version 2.1
 *	J-Sim source code can be downloaded from http://www.j-sim.zcu.cz/
 *
 */

package cz.zcu.fav.kiv.jsimexamples.ipc.messagepassing.synchronouswithknownsender;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;
import cz.zcu.fav.kiv.jsim.ipc.JSimMessage;
import cz.zcu.fav.kiv.jsim.ipc.JSimMessageBox;

/**
 * A simple process periodically receiving messages from a randomly selected process via a message box. If no message is available, the
 * receiver blocks until another process sends a message to the box. After every reading, the process sleeps for a while.
 */
public class ReceivingProcess extends JSimProcess
{
	public static final double LAMBDA = 1.0;

	private JSimMessageBox box;
	private JSimProcess sender1;
	private JSimProcess sender2;

	// ------------------------------------------------------------------------------------------------------------------------------------

	public ReceivingProcess(String name, JSimSimulation parent, JSimMessageBox messageBox, JSimProcess sendingProcess1, JSimProcess sendingProcess2) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException
	{
		super(name, parent);
		box = messageBox;
		sender1 = sendingProcess1;
		sender2 = sendingProcess2;
	} // constructor

	protected void life()
	{
		JSimMessage message;
		double sleepTime;
		JSimProcess selectedProcess;

		try
		{
			while (true)
			{
				if (JSimSystem.draw(0.5))
					selectedProcess = sender1;
				else
					selectedProcess = sender2;

				message(myParent.getCurrentTime() + " - " + getName() + ": Trying to receive some data from message box `" + box.getName() + "' from process `" + selectedProcess.getName() + "'.");
				message = receiveMessageWithBlocking(box, selectedProcess);
				message(myParent.getCurrentTime() + " - " + getName() + ": Running again.");
				message(myParent.getCurrentTime() + " - " + getName() + ": Received data: " + message.getData().toString());
				message(myParent.getCurrentTime() + " - " + getName() + ": Sender is `" + (message.getSender() == JSimMessage.UNKNOWN_SENDER ? "<unknown>" : message.getSender().getName()) + "'.");
				message(myParent.getCurrentTime() + " - " + getName() + ": Receiver is `" + (message.getReceiver() == JSimMessage.UNKNOWN_RECEIVER ? "<unknown>" : message.getReceiver().getName()) + "'.");

				sleepTime = JSimSystem.negExp(LAMBDA);
				message(myParent.getCurrentTime() + " - " + getName() + ": Going to sleep for " + sleepTime + ".");
				hold(sleepTime);
			} // while
		} // try
		catch (JSimException e)
		{
			e.printStackTrace();
			e.printComment(System.err);
		} // catch
	} // life

} // class ReceivingProcess
